from abc import ABC, abstractmethod
from enum import Enum, auto
from typing import Any, Dict

from httpx import Response

from checkbox_api.storage.simple import SessionStorage


class HTTPMethod(Enum):
    GET = auto()
    POST = auto()
    PUT = auto()


class AbstractMethod(ABC):
    method: HTTPMethod = HTTPMethod.GET

    @property
    @abstractmethod
    def uri(self) -> str:
        pass

    @property
    @abstractmethod
    def query(self):
        pass

    @property
    @abstractmethod
    def payload(self):
        pass

    @property
    @abstractmethod
    def headers(self):
        pass

    @abstractmethod
    def parse_response(self, storage: SessionStorage, response: Response):
        pass


class BaseMethod(AbstractMethod, ABC):
    @property
    def query(self):
        return {}

    @property
    def payload(self):
        return {}

    @property
    def headers(self):
        return {}

    def parse_response(self, storage: SessionStorage, response: Response):
        return response.json()


class PaginationMixin:
    def __init__(self, limit: int = 10, offset: int = 0):
        self.limit = limit
        self.offset = offset

    @property
    def query(self):
        query = {}
        if self.limit is not None:
            query["limit"] = self.limit
        if self.offset is not None:
            query["offset"] = self.offset
        return query

    def shift_next_page(self):
        self.offset += self.limit
        return self

    def shift_previous_page(self):
        self.offset -= self.limit
        return self

    def set_page(self, page: int):
        self.offset = self.limit * page

    def resolve_pagination(self, paginated_result: Dict[str, Any]):
        meta = paginated_result["meta"]
        self.offset = meta["offset"]
        self.limit = meta["limit"]
        return self

    @property
    def page(self):
        return self.offset // self.limit



import datetime

from httpx import Response

from checkbox_api.methods.base import BaseMethod, HTTPMethod, PaginationMixin
from checkbox_api.storage.simple import SessionStorage


class GetCashRegisters(PaginationMixin, BaseMethod):
    uri = "cash-registers"


class GetCashRegister(BaseMethod):
    def __init__(self, cash_register_id: str):
        self.cash_register_id = cash_register_id

    @property
    def uri(self) -> str:
        return f"cash-registers/{self.cash_register_id}"


class GetCashRegisterInfo(BaseMethod):
    uri: str = "cash-registers/info"

    def parse_response(self, storage: SessionStorage, response: Response):
        result = super().parse_response(storage=storage, response=response)
        storage.cash_register = result
        return result


class AskOfflineCodes(BaseMethod):
    uri = "cash-registers/ask-offline-codes"

    def __init__(self, count: int = 2000, sync: bool = False):
        self.count = count
        self.sync = sync

    @property
    def query(self):
        query = super().query
        query.update({"count": self.count, "sync": self.sync})
        return query


class GetOfflineCodes(BaseMethod):
    uri = "cash-registers/get-offline-codes"

    def __init__(self, count: int = 2000):
        self.count = count

    @property
    def query(self):
        query = super().query
        query.update({"count": self.count})
        return query


class GoOnline(BaseMethod):
    uri = "cash-registers/go-online"
    method = HTTPMethod.POST


class GoOffline(BaseMethod):
    uri = "cash-registers/go-offline"
    method = HTTPMethod.POST

    def __init__(self, go_offline_date: datetime.datetime, fiscal_code: str):
        self.go_offline_date = go_offline_date
        self.fiscal_code = fiscal_code

    @property
    def payload(self):
        payload = super().payload
        payload.update(
            {"go_offline_date": self.go_offline_date.isoformat(), "fiscal_code": self.fiscal_code}
        )
        return payload



import datetime

from httpx import Response

from checkbox_api.methods.base import BaseMethod, HTTPMethod, PaginationMixin
from checkbox_api.storage.simple import SessionStorage


class GetReports(PaginationMixin, BaseMethod):
    uri = "reports"


class GetReport(BaseMethod):
    def __init__(self, report_id: str):
        self.report_id = report_id

    @property
    def uri(self) -> str:
        return f"reports/{self.report_id}"


class CreateReport(BaseMethod):
    method = HTTPMethod.POST
    uri = "reports"


class GetReportVisualization(GetReport):
    def __init__(self, report_id: str, fmt: str = "text", **query):
        super().__init__(report_id=report_id)
        self.format = fmt
        self.params = query

    @property
    def query(self):
        query = super().query
        query.update(self.params)
        return query

    @property
    def uri(self) -> str:
        uri = super().uri
        return f"{uri}/{self.format}"

    def parse_response(self, storage: SessionStorage, response: Response):
        return response.content


class GetReportVisualizationText(GetReportVisualization):
    def __init__(self, report_id: str, width: int = 50):
        super().__init__(report_id=report_id, fmt="text", width=width)

    def parse_response(self, storage: SessionStorage, response: Response):
        result = super().parse_response(storage=storage, response=response)
        return result.decode()


class GetPeriodicalReportVisualizationText(BaseMethod):
    uri = "reports/periodical"

    def __init__(self, from_date: datetime.datetime, to_date: datetime.datetime, width: int = 50):
        super().__init__()
        self.from_date = from_date
        self.to_date = to_date
        self.width = width

    @property
    def query(self):
        query = super().query
        query.update(
            {
                "from_date": self.from_date.strftime("%Y-%m-%dT%H:%M:%S%z"),
                "to_date": self.to_date.strftime("%Y-%m-%dT%H:%M:%S%z"),
            }
        )
        if self.width:
            query["width"] = self.width
        return query

    def parse_response(self, storage: SessionStorage, response: Response):
        return response.content.decode()



from dataclasses import dataclass
from typing import Any, Dict, Optional

import jwt


@dataclass
class SessionStorage:
    def __init__(self, token: Optional[str] = None, license_key: Optional[str] = None):
        self.token = token
        self.license_key = license_key
        self.cashier = None
        self.cash_register = None
        self.shift = None

    @property
    def headers(self):
        headers = {}
        if self.token:
            headers["Authorization"] = f"Bearer {self.token}"
        if self.license_key:
            headers["X-License-Key"] = self.license_key
        return headers

    @property
    def token_data(self) -> Optional[Dict[str, Any]]:
        if self.token:
            return jwt.decode(self.token, verify=False)
        return None



import base64

from httpx import Response

from checkbox_api.methods.base import BaseMethod, HTTPMethod
from checkbox_api.storage.simple import SessionStorage

URI_PREFIX = "cashier/"


class _SignInMixin:
    method = HTTPMethod.POST

    def parse_response(self, storage: SessionStorage, response: Response):
        result = response.json()
        storage.token = result["access_token"]
        return result


class SignIn(_SignInMixin, BaseMethod):
    uri = URI_PREFIX + "signin"

    def __init__(self, login: str, password: str):
        self.login = login
        self.password = password

    @property
    def payload(self):
        return {"login": self.login, "password": self.password}


class SignInPinCode(_SignInMixin, BaseMethod):
    uri = SignIn.uri + "PinCode"

    def __init__(self, pin_code: str):
        self.pin_code = pin_code

    @property
    def payload(self):
        return {"pin_code": self.pin_code}


class SignInSignature(_SignInMixin, BaseMethod):
    uri = SignIn.uri + "Signature"

    def __init__(self, signature: bytes):
        self.signature = signature

    @property
    def payload(self):
        signature = base64.standard_b64encode(self.signature).decode()
        return {"signature": signature}


class GetMe(BaseMethod):
    method = HTTPMethod.GET
    uri = URI_PREFIX + "me"

    def parse_response(self, storage: SessionStorage, response: Response):
        result = super().parse_response(storage=storage, response=response)
        storage.cashier = result
        return result


class SignOut(BaseMethod):
    method = HTTPMethod.POST
    uri = URI_PREFIX + "signout"

    def parse_response(self, storage: SessionStorage, response: Response):
        result = super().parse_response(storage=storage, response=response)

        storage.shift = None
        storage.cashier = None
        storage.token = None

        return result


class GetActiveShift(BaseMethod):
    uri = URI_PREFIX + "shift"

    def parse_response(self, storage: SessionStorage, response: Response):
        result = super().parse_response(storage=storage, response=response)
        storage.shift = result
        return result



import datetime
import logging
import time
from typing import Any, Dict, List, Optional, Set

from httpcore import NetworkError
from httpx import Client, HTTPError, Timeout

from checkbox_api.client.base import BaseCheckBoxClient
from checkbox_api.consts import DEFAULT_REQUESTS_RELAX
from checkbox_api.exceptions import CheckBoxError, CheckBoxNetworkError, StatusException
from checkbox_api.methods import cash_register, cashier, receipts, shifts, transactions
from checkbox_api.methods.base import AbstractMethod, BaseMethod
from checkbox_api.storage.simple import SessionStorage

logger = logging.getLogger(__name__)


class CheckBoxClient(BaseCheckBoxClient):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        self._session = Client(
            proxies=self.proxy, timeout=Timeout(timeout=self.timeout), verify=self.verify_ssl
        )

    def emit(self, call: AbstractMethod, storage: Optional[SessionStorage] = None):
        storage = storage or self.storage

        url = f"{self.base_url}/api/v{self.api_version}/{call.uri}"
        try:
            response = self._session.request(
                method=call.method.name,
                url=url,
                timeout=self.timeout,
                params=call.query,
                headers={**storage.headers, **call.headers, **self.client_headers},
                json=call.payload,
            )
        except HTTPError as e:
            raise CheckBoxError(e)
        except NetworkError as e:
            raise CheckBoxNetworkError(e)

        self._check_response(response=response)
        result = call.parse_response(storage=storage, response=response)

        return result

    def refresh_info(self, storage: Optional[SessionStorage] = None):
        storage = storage or self.storage

        self(cashier.GetMe(), storage=storage)
        self(cashier.GetActiveShift(), storage=storage)
        if storage.license_key:
            self(cash_register.GetCashRegisterInfo())

    def authenticate(
        self,
        login: str,
        password: str,
        license_key: Optional[str] = None,
        storage: Optional[SessionStorage] = None,
    ) -> None:
        self._set_license_key(storage=storage, license_key=license_key)
        self(cashier.SignIn(login=login, password=password), storage=storage)
        self.refresh_info(storage=storage)

    def authenticate_signature(
        self,
        signature: bytes,
        license_key: Optional[str] = None,
        storage: Optional[SessionStorage] = None,
    ) -> None:
        self._set_license_key(storage=storage, license_key=license_key)
        self(cashier.SignInSignature(signature=signature), storage=storage)
        self.refresh_info(storage=storage)

    def authenticate_pin_code(
        self,
        pin_code: str,
        license_key: Optional[str] = None,
        storage: Optional[SessionStorage] = None,
    ) -> None:
        self._set_license_key(storage=storage, license_key=license_key)
        self(cashier.SignInPinCode(pin_code=pin_code), storage=storage)
        self.refresh_info(storage=storage)

    def authenticate_token(
        self,
        token: str,
        license_key: Optional[str] = None,
        storage: Optional[SessionStorage] = None,
    ) -> None:
        storage = storage or self.storage
        self._set_license_key(storage=storage, license_key=license_key)
        storage.token = token
        self.refresh_info(storage=storage)

    def get_shifts(
        self,
        statuses: Optional[List[str]] = None,
        limit: int = 10,
        storage: Optional[SessionStorage] = None,
    ):
        get_shift = shifts.GetShifts(statuses=statuses, limit=limit)
        while (shifts_result := self(get_shift, storage=storage))["results"]:
            get_shift.resolve_pagination(shifts_result).shift_next_page()
            yield from shifts_result["results"]

    def _wait_status(
        self,
        method: BaseMethod,
        expected_value: Set[Any],
        field: str = "status",
        relax: int = DEFAULT_REQUESTS_RELAX,
        timeout: Optional[int] = None,
        storage: Optional[SessionStorage] = None,
    ):
        logger.info("Wait until %r will be changed to one of %s", field, expected_value)
        initial = time.monotonic()
        while (result := self(method, storage=storage))[field] not in expected_value:
            if timeout is not None and time.monotonic() > initial + timeout:
                logger.error("Status did not changed in required time")
                break
            time.sleep(relax)

        if result[field] not in expected_value:
            raise ValueError(
                f"Object did not change field {field!r} "
                f"to one of expected values {expected_value} (actually {result[field]!r}) "
                f"in {time.monotonic() - initial:.3f} seconds"
            )

        logger.info(
            "Status changed in %.3f seconds to %r", time.monotonic() - initial, result[field],
        )
        return result

    def create_shift(
        self,
        relax: int = DEFAULT_REQUESTS_RELAX,
        timeout: Optional[int] = None,
        storage: Optional[SessionStorage] = None,
    ):
        storage = storage or self.storage
        self.refresh_info(storage=storage)
        if storage.shift is not None:
            logger.info(
                "Shift is already opened %s in status %s",
                storage.shift["id"],
                storage.shift["status"],
            )
            shift = storage.shift
        else:
            shift = self(shifts.CreateShift(), storage=storage)
            logger.info("Created shift %s", shift["id"])

        if shift["status"] == "OPENED":
            return shift

        shift = self._wait_status(
            shifts.GetShift(shift_id=shift["id"]),
            storage=storage,
            relax=relax,
            field="status",
            expected_value={"OPENED", "CLOSED"},
            timeout=timeout,
        )
        if shift["status"] == "CLOSED":
            initial_transaction = shift["initial_transaction"]
            raise StatusException(
                "Shift can not be opened in due to transaction status moved to "
                f"{initial_transaction['status']!r}: {initial_transaction['response_status']!r} "
                f"{initial_transaction['response_error_message']!r}"
            )
        return shift

    def close_shift(
        self,
        relax: int = DEFAULT_REQUESTS_RELAX,
        timeout: Optional[int] = None,
        storage: Optional[SessionStorage] = None,
    ):
        storage = storage or self.storage
        self.refresh_info(storage=storage)
        if storage.shift is None:
            logger.info("Shift is already closed")
            return storage.shift

        shift = self(shifts.CloseShift(), storage=storage)
        logger.info("Trying to close shift %s", shift["id"])

        shift = self._wait_status(
            shifts.GetShift(shift_id=shift["id"]),
            storage=storage,
            relax=relax,
            field="status",
            expected_value={"OPENED", "CLOSED"},
            timeout=timeout,
        )
        if shift["status"] == "OPENED":
            closing_transaction = shift["closing_transaction"]
            raise StatusException(
                "Shift can not be closed in due to transaction status moved to "
                f"{closing_transaction['status']!r}: {closing_transaction['response_status']!r} "
                f"{closing_transaction['response_error_message']!r}"
            )
        return shift["z_report"]

    def create_receipt(
        self,
        receipt: Optional[Dict[str, Any]] = None,
        relax: int = DEFAULT_REQUESTS_RELAX,
        timeout: Optional[int] = None,
        storage: Optional[SessionStorage] = None,
        **payload,
    ):
        receipt = self(receipts.CreateReceipt(receipt, **payload), storage=storage)
        logger.info("Trying create receipt %s", receipt["id"])

        shift = self._wait_status(
            receipts.GetReceipt(receipt_id=receipt["id"]),
            storage=storage,
            relax=relax,
            field="status",
            expected_value={"DONE", "ERROR"},
            timeout=timeout,
        )
        if shift["status"] == "ERROR":
            initial_transaction = shift["transaction"]
            raise StatusException(
                "Receipt can not be created in due to transaction status moved to "
                f"{initial_transaction['status']!r}: {initial_transaction['response_status']!r} "
                f"{initial_transaction['response_error_message']!r}"
            )
        return shift

    def create_external_receipt(
        self,
        receipt: Optional[Dict[str, Any]] = None,
        relax: int = DEFAULT_REQUESTS_RELAX,
        timeout: Optional[int] = None,
        storage: Optional[SessionStorage] = None,
        **payload,
    ):
        receipt = self(receipts.AddExternal(receipt, **payload), storage=storage)
        logger.info("Trying to create external receipt %s", receipt["id"])

        shift = self._wait_status(
            receipts.GetReceipt(receipt_id=receipt["id"]),
            storage=storage,
            relax=relax,
            field="status",
            expected_value={"DONE", "ERROR"},
            timeout=timeout,
        )
        if shift["status"] == "ERROR":
            initial_transaction = shift["transaction"]
            raise StatusException(
                "Receipt can not be created in due to transaction status moved to "
                f"{initial_transaction['status']!r}: {initial_transaction['response_status']!r} "
                f"{initial_transaction['response_error_message']!r}"
            )
        return shift

    def create_service_receipt(
        self,
        payment: Dict[str, Any],
        id: Optional[str] = None,
        fiscal_code: Optional[str] = None,
        fiscal_date: Optional[datetime.datetime] = None,
        relax: int = DEFAULT_REQUESTS_RELAX,
        timeout: Optional[int] = None,
        storage: Optional[SessionStorage] = None,
    ):
        receipt = self(
            receipts.CreateServiceReceipt(
                payment=payment, id=id, fiscal_code=fiscal_code, fiscal_date=fiscal_date
            ),
            storage=storage,
        )
        logger.info("Trying to create receipt %s", receipt["id"])

        shift = self._wait_status(
            receipts.GetReceipt(receipt_id=receipt["id"]),
            storage=storage,
            relax=relax,
            field="status",
            expected_value={"DONE", "ERROR"},
            timeout=timeout,
        )
        if shift["status"] == "ERROR":
            initial_transaction = shift["transaction"]
            raise StatusException(
                "Receipt can not be created in due to transaction status moved to "
                f"{initial_transaction['status']!r}: {initial_transaction['response_status']!r} "
                f"{initial_transaction['response_error_message']!r}"
            )
        return shift

    def get_offline_codes(self, count: int = 2000):
        logger.info("Ask offline codes (count=%d)", count)
        self(cash_register.AskOfflineCodes(count=count, sync=True))
        logger.info("Load offline codes...")
        codes = self(cash_register.GetOfflineCodes(count=count))
        return [item["fiscal_code"] for item in codes]

    def wait_transaction(
        self,
        transaction_id: str,
        relax: int = DEFAULT_REQUESTS_RELAX,
        timeout: Optional[int] = None,
        storage: Optional[SessionStorage] = None,
    ):
        transaction = self._wait_status(
            transactions.GetTransaction(transaction_id=transaction_id),
            relax=relax,
            timeout=timeout,
            storage=storage,
            field="status",
            expected_value={"DONE", "ERROR"},
        )
        if transaction["status"] == "ERROR":
            raise StatusException(
                f"Transaction status moved to {transaction['status']!r} "
                f"and tax status {transaction['response_status']!r} "
                f"with message {transaction['response_error_message']!r}"
            )
        return transaction



from typing import List, Optional

from httpx import Response

from checkbox_api.methods.base import BaseMethod, HTTPMethod, PaginationMixin
from checkbox_api.storage.simple import SessionStorage


class GetShifts(PaginationMixin, BaseMethod):
    uri = "shifts"

    def __init__(self, statuses: Optional[List[str]] = None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.statuses = statuses

    @property
    def query(self):
        query = super().query
        if self.statuses is not None:
            query["statuses"] = self.statuses
        return query


class CreateShift(BaseMethod):
    method = HTTPMethod.POST
    uri = "shifts"

    def parse_response(self, storage: SessionStorage, response: Response):
        result = super().parse_response(storage=storage, response=response)
        storage.shift = result
        return result


class CloseShift(BaseMethod):
    method = HTTPMethod.POST
    uri = "shifts/close"

    def parse_response(self, storage: SessionStorage, response: Response):
        result = super().parse_response(storage=storage, response=response)
        storage.shift = result
        return result


class GetShift(BaseMethod):
    def __init__(self, shift_id: str):
        self.shift_id = shift_id

    @property
    def uri(self) -> str:
        return f"shifts/{self.shift_id}"

    def parse_response(self, storage: SessionStorage, response: Response):
        result = super().parse_response(storage=storage, response=response)
        if storage.shift and storage.shift["id"] == result["id"]:
            storage.shift = result
        return result



from typing import Any, Dict


class CheckBoxError(Exception):
    pass


class CheckBoxNetworkError(CheckBoxError):
    pass


class CheckBoxAPIError(CheckBoxError):
    def __init__(self, status: int, content: Dict[str, Any]):
        self.status = status
        self.content = content
        self.message = content.get("message", content)

    def __str__(self):
        return f"{self.message} [status={self.status}]"


class CheckBoxAPIValidationError(CheckBoxAPIError):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.detail = self.content.get("detail", [])

    def __str__(self):
        validations = []
        for item in self.detail:
            location = " -> ".join(map(str, item["loc"]))
            error_type = item["type"]
            description = item["msg"]
            validations.append(f"{location}:\n    {description} (type={error_type})")
        validations_str = "\n".join(validations)
        return f"{self.message} [status={self.status}]\n{validations_str}"


class StatusException(CheckBoxError):
    pass



from typing import List, Optional

from checkbox_api.methods.base import BaseMethod, PaginationMixin


class GetTransactions(PaginationMixin, BaseMethod):
    uri = "transactions"

    def __init__(
        self, status: Optional[List[str]] = None, type: Optional[List[str]] = None, *args, **kwargs
    ):
        super().__init__(*args, **kwargs)
        self.status = status
        self.type = type

    @property
    def query(self):
        query = super().query
        if self.status is not None:
            query["status"] = self.status
        if self.type is not None:
            query["type"] = self.type
        return query


class GetTransaction(BaseMethod):
    def __init__(self, transaction_id: str):
        self.transaction_id = transaction_id

    @property
    def uri(self) -> str:
        return f"transactions/{self.transaction_id}"



import logging
from abc import ABC, abstractmethod
from typing import Any, Dict, Optional, Union

from httpx import Response

from checkbox_api import __version__
from checkbox_api.consts import API_VERSION, BASE_API_URL, DEFAULT_REQUEST_TIMEOUT
from checkbox_api.exceptions import CheckBoxAPIError, CheckBoxAPIValidationError, CheckBoxError
from checkbox_api.methods.base import AbstractMethod
from checkbox_api.storage.simple import SessionStorage

logger = logging.getLogger(__name__)


class BaseCheckBoxClient(ABC):
    def __init__(
        self,
        base_url: str = BASE_API_URL,
        requests_timeout: int = DEFAULT_REQUEST_TIMEOUT,
        proxy: Optional[Union[str, Dict[str, str]]] = None,
        verify_ssl: bool = True,
        api_version: str = API_VERSION,
        storage: Optional[SessionStorage] = None,
        client_name: str = "CheckBox Python SDK",
        client_version: str = __version__,
    ) -> None:
        self.base_url = base_url
        self.api_version = api_version
        self.timeout = requests_timeout
        self.proxy = proxy
        self.verify_ssl = verify_ssl
        self.storage = storage or SessionStorage()
        self.client_name = client_name
        self.client_version = client_version

    @property
    def client_headers(self) -> Dict[str, Any]:
        headers = {"X-Client-Name": self.client_name, "X-Client-Version": self.client_version}
        return headers

    @abstractmethod
    def emit(self, storage: SessionStorage, method: AbstractMethod):
        pass

    def __call__(self, *args, **kwargs):
        return self.emit(*args, **kwargs)

    @classmethod
    def _check_response(cls, response: Response):
        if response.status_code >= 500:
            raise CheckBoxError(
                f"Failed to make request [status={response.status_code}, text={response.text!r}]"
            )
        if response.status_code == 422:
            raise CheckBoxAPIValidationError(status=response.status_code, content=response.json())
        if response.status_code >= 400:
            raise CheckBoxAPIError(status=response.status_code, content=response.json())

    def _set_license_key(
        self, storage: Optional[SessionStorage], license_key: Optional[str]
    ) -> None:
        if license_key is None:
            return
        storage = storage or self.storage
        storage.license_key = license_key



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['checkbox_api',
 'checkbox_api.client',
 'checkbox_api.methods',
 'checkbox_api.storage']

package_data = \
{'': ['*']}

install_requires = \
['PyJWT>=1.7.1,<2.0.0', 'httpx>=0.13.3,<0.14.0']

setup_kwargs = {
    'name': 'checkbox-api',
    'version': '0.1.16',
    'description': 'checkbox.in.ua Transactional Processing API Client',
    'long_description': None,
    'author': 'Oleksandr Onufriichuk',
    'author_email': 'oleksandr.onufriichuk@itvaan.com.ua',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import datetime
from typing import Any, Dict, Optional

from httpx import Response

from checkbox_api.methods.base import BaseMethod, HTTPMethod, PaginationMixin
from checkbox_api.storage.simple import SessionStorage


class GetReceipts(PaginationMixin, BaseMethod):
    uri = "receipts"


class GetReceipt(BaseMethod):
    def __init__(self, receipt_id: str):
        self.receipt_id = receipt_id

    @property
    def uri(self) -> str:
        return f"receipts/{self.receipt_id}"


class CreateReceipt(BaseMethod):
    method = HTTPMethod.POST
    uri = "receipts/sell"

    def __init__(
        self, receipt: Optional[Dict] = None, **payload,
    ):
        if receipt is not None and payload is not None:
            raise ValueError("'receipt' and '**payload' can not be passed together")
        self.receipt = receipt or payload

    @property
    def payload(self):
        payload = super().payload
        payload.update(self.receipt)
        return payload

    def parse_response(self, storage: SessionStorage, response: Response):
        result = super().parse_response(storage=storage, response=response)
        storage.shift = result["shift"]
        return result


class CreateServiceReceipt(BaseMethod):
    method = HTTPMethod.POST
    uri = "receipts/service"

    def __init__(
        self,
        payment: Dict[str, Any],
        id: Optional[str] = None,
        fiscal_code: Optional[str] = None,
        fiscal_date: Optional[datetime.datetime] = None,
    ):
        self.payment = payment
        self.id = id
        self.fiscal_code = fiscal_code
        self.fiscal_date = fiscal_date

    @property
    def payload(self):
        payload = super().payload
        payload["payment"] = self.payment
        if self.id:
            payload["id"] = self.id
        if self.fiscal_code:
            payload["fiscal_code"] = self.fiscal_code
        if self.fiscal_date:
            payload["fiscal_date"] = self.fiscal_date
        return payload

    def parse_response(self, storage: SessionStorage, response: Response):
        result = super().parse_response(storage=storage, response=response)
        storage.shift = result["shift"]
        return result


class GetReceiptVisualization(GetReceipt):
    def __init__(self, receipt_id: str, fmt: str = "text", **query):
        super().__init__(receipt_id=receipt_id)
        self.format = fmt
        self.params = query

    @property
    def query(self):
        query = super().query
        query.update(self.params)
        return query

    @property
    def uri(self) -> str:
        uri = super().uri
        return f"{uri}/{self.format}"

    def parse_response(self, storage: SessionStorage, response: Response):
        return response.content


class GetReceiptVisualizationText(GetReceiptVisualization):
    def __init__(self, receipt_id: str, width: int = 50):
        super().__init__(receipt_id=receipt_id, fmt="text", width=width)

    def parse_response(self, storage: SessionStorage, response: Response):
        result = super().parse_response(storage=storage, response=response)
        return result.decode()


class GetReceiptVisualizationHtml(GetReceiptVisualization):
    def __init__(self, receipt_id: str):
        super().__init__(receipt_id=receipt_id, fmt="html")


class GetReceiptVisualizationPdf(GetReceiptVisualization):
    def __init__(self, receipt_id: str):
        super().__init__(receipt_id=receipt_id, fmt="pdf")


class GetReceiptVisualizationQrCode(GetReceiptVisualization):
    def __init__(self, receipt_id: str):
        super().__init__(receipt_id=receipt_id, fmt="qrcode")


class AddExternal(BaseMethod):
    method = HTTPMethod.POST
    uri = "receipts/add-external"

    def __init__(
        self, receipt: Optional[Dict] = None, **payload,
    ):
        if receipt is not None and payload is not None:
            raise ValueError("'receipt' and '**payload' can not be passed together")
        self.receipt = receipt or payload

    @property
    def payload(self):
        return self.receipt
